------------------------------------------------------ Setup
local function SetDVars(p, dvars) -- Set up values
	if(p.dvars == nil) then
		p.dvars = {}
	end
    if(dvars) or(p.dvars.energy == 0) or(p.dvars.hull == 0) then
        p.dvars.energy = 350
		p.dvars.hull = 50
    end
	p.dvars.prevrings1 = p.rings
    p.dvars.prevrings2 = p.rings
end

addHook("PlayerThink", function(p) -- Check if values exist
	if p.mo.skin == "drone" then
		if(p.dvars == nil) then
			SetDVars(p, true)
		end
    end
end)

addHook("MapLoad", function() -- Reset on map change
	for p in players.iterate() do
		SetDVars(p, true)
	end
end)
------------------------------------------------------ Health
addHook("PlayerThink", function(p)
    if(p.health == 0) then
        p.dvars.hull = 50 -- Reset!
    end
end)
addHook("PlayerThink", function(p) -- Ring healing
    if(p.rings - p.dvars.prevrings2 == 1) then
        p.dvars.hull = $1 + 1
    if(p.dvars.hull > 50) then
        p.dvars.hull = 50
    end
        elseif(p.dvars.hull < 50) and(p.rings > p.dvars.prevrings2) then
         p.dvars.hull = $1 + p.rings - p.dvars.prevrings2
        if(p.dvars.hull > 50) then
            p.dvars.hull = 50
        end
    end
    p.dvars.prevrings2 = p.rings
end)
------------------------------------------ Damage
addHook("MobjDamage", function(mobj, inflictor, source, damage)
    if mobj.skin == "drone" and not (mobj.player.powers[pw_carry] == CR_NIGHTSMODE) then
        if(mobj.player.powers[pw_shield]) then -- Have a shield?
		    P_RemoveShield(mobj.player) -- It's gone.
		    P_PlayerFlagBurst(mobj.player)
        else
            P_PlayerFlagBurst(mobj.player) -- Damage
            mobj.player.dvars.hull = $1 - 5
        end
        if(mobj.health <= 0) or (mobj.player.dvars.hull <= 0) then
			P_KillMobj(mobj) -- RIP
			mobj.health = 0
        end
        if(mobj.health > 0) then	-- Still Alive
		    mobj.player.powers[pw_flashing] = 50 -- I'm invinicble!
		    P_DoPlayerPain(mobj.player, source, inflictor) -- Ouch
	    end
    end
    return true
end, MT_PLAYER)
addHook("PlayerThink", function(p)
    if p.mo.skin == "drone" then
------------------------------------------------------ Battery charging
		if (p.mo.eflags & MFE_JUSTHITFLOOR) > 0 and p.dvars.energy < 262 then -- Did we just land and have less than 175 capacity?
			p.dvars.energy = 262 -- 3/4 full now
		end
        if P_IsObjectOnGround(p.mo) then -- Are we on the ground?
			if p.dvars.energy < 350 then -- Don't go above 350
           		p.dvars.energy = $1 + 1 -- Charge!
			end
        end
    end
------------------------------------------------------ Ring energy
	if(p.rings - p.dvars.prevrings1 == 1) then  -- Check collected rings
		if p.dvars.energy < 350 then
			p.dvars.energy = $1 + 35 -- More power to you
		end
	end
p.dvars.prevrings1 = p.rings
------------------------------------------------------ Abilities
	if p.mo.skin == "drone" and not (p.powers[pw_carry] == CR_NIGHTSMODE) then
		if p.dvars.energy > 5 then
			if (p.cmd.buttons & BT_JUMP) then
				P_SetObjectMomZ(p.mo, 10*FRACUNIT, false) -- Flight
				p.dvars.energy = $1 - 1
			end
			if (p.cmd.buttons & BT_SPIN) then
				if not P_IsObjectOnGround(p.mo) then
					P_SetObjectMomZ(p.mo, -10*FRACUNIT, false) -- Descent
				end
			end
			if (p.cmd.buttons & BT_CUSTOM1) then
				local truespeed = FixedMul(p.mo.scale, p.actionspd)
				P_InstaThrust(p.mo, p.mo.angle, truespeed) -- Boost
				P_SpawnThokMobj(p)
				p.dvars.energy = $1 - 1
				p.drawangle = p.mo.angle
			end
			if (p.cmd.buttons & BT_FIRENORMAL) then -- Lasers
				local dronelaser1 = P_SpawnMobj(p.mo.x, p.mo.y, p.mo.z + p.mo.height, MT_TURRETLASER) -- Top one
				dronelaser1.target = p.mo
				P_InstaThrust(dronelaser1, p.mo.angle, (100*FRACUNIT))
				local dronelaser2 = P_SpawnMobj(p.mo.x, p.mo.y, p.mo.z + (p.mo.height-30*FRACUNIT), MT_TURRETLASER) -- Bottom one
				dronelaser2.target = p.mo
				P_InstaThrust(dronelaser2, p.mo.angle, (100*FRACUNIT))
				p.dvars.energy = $1 - 1
				p.drawangle = p.mo.angle
			end
			if (p.cmd.buttons & BT_CUSTOM2) then
				P_SetObjectMomZ(p.mo, 0*FRACUNIT, false) -- Hover
				p.dvars.energy = $1 - 1
			end
				p.powers[pw_underwater] = 1000 -- Air
				p.powers[pw_spacetime] = 1000
				p.pflags = $ | PF_CANCARRY -- Can carry
			end
		end
	end)
------------------------------------------------------ HUD
local function d_hud(v, p) -- Player check
	if not(p.mo) or(p.powers[pw_carry] == CR_NIGHTSMODE) or not(p.mo.skin == "drone") then
		return
	end
		v.drawString (100, 125, p.dvars.energy, V_SNAPTOBOTTOM)
		v.drawString (200, 125, p.dvars.hull, V_SNAPTOBOTTOM)
		v.draw (125, 125, v.cachePatch("BATT"), V_SNAPTOBOTTOM)
		v.draw (190, 125, v.cachePatch("HULL"), V_SNAPTOBOTTOM)
	end
hud.add(d_hud)